using System;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using QTPlugin.Interop;


namespace QTPlugin
{

	/// <summary>
	/// Base plugin interface.
	/// </summary>
	public interface IPluginClient
	{
		/// <summary>
		/// Called by QTTabBar rigth after inheriting class constructed.
		/// Do initialization of plugin here.
		/// Note that PluginType.Background type plugin may be instantialized on Option window of QTTabBar.
		/// Note that interactive toolbar item is not created yet.
		/// </summary>
		/// 
		/// <param name="pluginServer">
		/// IPluginServer interface of QTTabBar.
		/// Hold and use this parameter to command QTTabBar, get information of explorer or tabbar, etc.
		/// </param>
		/// 
		/// <param name="shellBrowser">
		/// IShellBrowser interface of explorer window.
		/// Implementer can hold the instance for the lifetime of this plugin,
		/// and can use this parameter to get ISellView, IFolderView, etc.
		/// There's no need to call Marshal.ReleaseComObject() to this IShellBrowser interface.
		/// </param>
		void Open( IPluginServer pluginServer, IShellBrowser shellBrowser );
		

		/// <summary>
		/// QTTabBar notifies closing event.
		/// 
		/// If endCode is NOT EndCode.Hidden, QTTabBar loses the reference to this plugin instance after this call.
		/// Clean up managed/unmanaged resources here if any.
		/// 
		/// EndCode.Hidden is passed only when this has PluginType.Background attribute and toolbar item is disabled by user.
		/// In this case, the plugin is still alive and can interact with user. 
		/// 
		/// See EndCode enumeration.
		/// </summary>
		/// <param name="endCode"></param>
		void Close( EndCode endCode );


		/// <summary>
		/// To allow users to set shortcut keys, 
		/// set action names and return true.
		/// members of actions will be shown in the Option -> "Shortcut Keys".
		/// 
		/// Return false if no need of shortcut keys.
		/// </summary>
		/// <param name="actions"></param>
		/// <returns></returns>
		bool QueryShortcutKeys( out string[] actions );


		/// <summary>
		/// Gets whether the plugin has options.
		/// </summary>
		bool HasOption
		{
			get;
		}


		/// <summary>
		/// Called when "Plugin Option" button pressed.
		/// About saving settings, see Instructions.txt.
		/// </summary>
		void OnOption();


		/// <summary>
		/// Called when user click the registered menu item.
		/// To register, use <c>IPluginServer.RegisterMenu()</c>.
		/// </summary>
		/// <param name="menuType">Type of menu registered. Tab or Bar.</param>
		/// <param name="menuText">Identifier text of the clicked menu item.</param>
		/// <param name="tab">
		/// If <paramref name="menuType"/> is MenuType.Tab, the tab which user open a context menu.
		/// If <paramref name="menuType"/> is MenuType.Bar, this parameter is null.
		/// </param>
		void OnMenuItemClick( MenuType menuType, string menuText, ITab tab );
		

		/// <summary>
		/// Called when shortcut keys pressed.
		/// </summary>
		/// <param name="index">
		/// Index of shortcut key registered.
		/// This is an index in the parameter "actions" passed to QueryShortcutKeys().
		/// </param>
		void OnShortcutKeyPressed( int index );

	}


	#region ------- Button plugin interfaces -------

	/// <summary>
	/// Inherit this interface if want a simple button item.
	/// QTTabBar notifies only when clicked.
	/// </summary>
	public interface IBarButton : IPluginClient
	{
		/// <summary>
		/// Called every time the button item of this plugin is about to be added to the explorer toolbar.
		/// Note that ButtonBar Option window recreates all items when its options changed.
		/// </summary>
		void InitializeItem();


		/// <summary>
		/// Return button image.
		/// if fLarge is true, return 24x24 image.
		/// if fLarge is false, return 16x16 image.
		/// 
		/// The call usually follows IBarButton.InitializeItem().
		/// Also called when plugin calls IPluginServer.UpdateItem() with parameter fRefreshImage true.
		/// </summary>
		Image GetImage( bool fLarge );


		/// <summary>
		/// Called by QTTabBar when the button clicked.
		/// </summary>
		void OnButtonClick();


		/// <summary>
		/// Gets display text.
		/// </summary>
		string Text
		{
			get;
		}


		/// <summary>
		/// Gets text lable mode.
		/// Return true if want to show text label when user set "Selective text" of ButtonBar.
		/// </summary>
		bool ShowTextLabel
		{
			get;
		}

	}

	/// <summary>
	/// This interface offers ToolStripDropDownButton and ToolStripSplitButton.
	/// </summary>
	public interface IBarDropButton : IBarButton
	{
		/// <summary>
		/// Called by QTTabBar, every time the dropdown menu is opening.
		/// 
		/// If you want to make DropDownMenu by yourself, use IBarCustomItem.
		/// </summary>
		/// <param name="menu">ToolStripDropDownMenu instance. Add items to <c>menu.Items</c>. </param>
		void OnDropDownOpening( ToolStripDropDownMenu menu );


		/// <summary>
		/// Called by QTTabBar, when dropdown menu item clicked.
		/// </summary>
		/// <param name="item">The item clicked.</param>
		/// <param name="mouseButton">Mouse button pressed. Left or Right.</param>
		void OnDropDownItemClick( ToolStripItem item, MouseButtons mouseButton );


		/// <summary>
		/// Gets whether the plugin button is a splitbutton.
		/// Called when QTButtonBar creating DropButton item.
		/// If returns false, IBarButton.OnButtonClick method will never be called.
		/// </summary>
		bool IsSplitButton
		{
			get;
		}

	}

	/// <summary>
	/// Inherit this interface if want a custom toolbar item.
	/// </summary>
	public interface IBarCustomItem : IPluginClient
	{
		/// <summary>
		/// Return an item that inherits ToolStripItem class.
		/// 
		/// Note that ToolStripItem.Tag property is used for inner management so it will be overwritten even if set.
		/// Note that Button Bar Option window recreates all items when its options changed.
		/// </summary>
		/// <param name="fLarge">true if current toolbar button icon is large.</param>
		/// <param name="displayStyle">current bar style.</param>
		/// <returns>ToolStripItem instance.</returns>
		ToolStripItem CreateItem( bool fLarge, DisplayStyle displayStyle );

	}


	#endregion


	/// <summary>
	/// Inherit this interface if want to create a custom search-box filter engine.
	/// 
	/// This interface is to convert a string user typed in search-box to Regular Expression for filtering file itmes.
	/// 
	/// One instance can be activated in system. QTTabBar uses first valid instance found and ignores the rest.
	/// Plugin that inherits this interface must be PluginType.Background.
	/// 
	/// ( This engine might be used for incremental search in Folder View of explorer in the future. )
	/// </summary>
	public interface IFilter : IPluginClient
	{
		/// <summary>
		/// Retrieves a Regular Expression object.
		/// 
		/// </summary>
		/// <param name="strQuery">In. string that user typed in Search-Box of QTButtonBar.</param>
		/// <param name="re">Out. regular expression object.</param>
		/// <returns>if returns false, QTTabBar uses its default filtering function.</returns>
		bool QueryRegex( string strQuery, out Regex re );
	}

	/// <summary>
	/// Inherit this interface if want to create a custom search-box filter engine.
	/// 
	/// This interface makes a decision on whether a file item is filtered or not, passed Regular Expression object
	/// that is created by QTTabBar(default) or IFilter instance. 
	/// 
	/// Implementor can use IShellFolder interface to get informations of file passed as pIDLChild.
	/// 
	/// One instance can be activated in system. QTTabBar uses first valid instance found and ignores the rest.
	/// Plugin that inherits this interface must be PluginType.Background.
	/// </summary>
	public interface IFilterCore : IPluginClient
	{
		/// <summary>
		/// Return true if reQuery matches the item identified by pIDLChild.
		/// </summary>
		/// 
		/// <param name="shellFolder">
		/// IShellFolder interface that specifies the folder where the search is performed.
		/// You can cast this to IShellFolder2 if the folder implements it.
		/// 
		/// There's no need to call Marshal.ReleaseComObject() to this interface.
		/// Do not hold this beyond method scope.
		/// </param>
		/// 
		/// <param name="pIDLChild">
		/// Pointer to an item's ITEMIDLIST structure, relative to parent folder.
		/// </param>
		/// 
		/// <param name="reQurey">
		/// Regular Expression object that is created by QTTabBar(default) or IFilter instance.
		/// </param>
		/// 
		/// <returns></returns>
		bool IsMatch( IShellFolder shellFolder, IntPtr pIDLChild, Regex reQurey );
	}

	///// <summary>
	///// This interface is for text preview tip.
	///// 
	///// </summary>
	//public interface IEncodingDetector : IPluginClient
	//{
	//    /// <summary>
	//    /// currently not called.
	//    /// </summary>
	//    /// <param name="data"></param>
	//    /// <returns></returns>
	//    EndCode GetEncode( byte[] data );
	//}



}