using System;
using System.Drawing;
using System.Diagnostics;
using QTPlugin.Interop;


namespace QTPlugin
{
	/// <summary>
	/// QTTabBar implements this interface.
	/// </summary>
	public interface IPluginServer
	{

		#region Events


		/// <summary>
		/// Occurs when current active tab has changed.
		/// </summary>
		event PluginEventHandler TabChanged;

		/// <summary>
		/// Occurs when a new tab added to tabbar.
		/// </summary>
		event PluginEventHandler TabAdded;

		/// <summary>
		/// Occurs when a tab removed from tabbar.
		/// </summary>
		event PluginEventHandler TabRemoved;

		/// <summary>
		/// Occurs when settings of QTTabBar have changed.
		/// </summary>
		event PluginEventHandler SettingsChanged;


		/// <summary>
		/// Occurs when explorer navigation is completed.
		/// </summary>
		event PluginEventHandler NavigationComplete;
		
		/// <summary>
		/// Occurs when the selection state of any item or items in the folder view has changed.
		/// </summary>
		event PluginEventHandler SelectionChanged;

		/// <summary>
		/// Occurs when Explorer window maximized/minimized.
		/// </summary>
		event PluginEventHandler ExplorerStateChanged;

		/// <summary>
		/// Occurs when the mouse pointer enters the tabBar. 
		/// </summary>
		event EventHandler MouseEnter;

		/// <summary>
		/// Occurs when the mouse pointer moves and points a tab.
		/// </summary>
		event PluginEventHandler PointedTabChanged;

		/// <summary>
		/// Occurs when the mouse pointer leaves the TabBar.
		/// </summary>
		event EventHandler MouseLeave;


		#endregion



		/// <summary>
		/// Registers or unregisters a menu item to QTTabBar menu.
		/// The menus registered are automatically removed on unloading.
		/// </summary>
		/// <param name="pluginClient">Plugin to register/unregister.</param>
		/// <param name="menuType">Tab, Bar, or Both</param>
		/// <param name="menuText">Menu item text</param>
		/// <param name="fRegister"></param>
		void RegisterMenu( IPluginClient pluginClient, MenuType menuType, string menuText, bool fRegister );


		/// <summary>
		/// Updates ToolStripItem state.
		/// This reads IBarButton.Text.
		/// 
		/// be careful not to make an infinite loop by calling this method in IBarButton.Text or IBarButton.GetImage().
		/// </summary>
		/// <param name="barItem">Plugin to update.</param>
		/// <param name="fEnabled">ToolStripItem.Enabled value.</param>
		/// <param name="fRefreshImage">if true IPluginServer calls IBarButton.GetImage() and refresh image.</param>
		void UpdateItem( IBarButton barItem, bool fEnabled, bool fRefreshImage );


		/// <summary>
		/// Executes a command on QTTabBar.
		/// </summary>
		/// <param name="command"></param>
		/// <param name="arg">
		/// set this parameter if the command requires arguments, or set null otherwise.
		/// see Commands enumeration.
		/// </param>
		/// <returns></returns>
		bool ExecuteCommand( Commands command, object arg );


		/// <summary>
		/// Gets plugin localized strings.
		/// see "Localizing" in Instructions.txt.
		/// </summary>
		/// <param name="pluginClient"></param>
		/// <param name="count">
		/// Desired number of strings in returned array.
		/// If not matches the number of lines in language file, this returns false and arrStrings is null.
		/// </param>
		/// <param name="arrStrings"></param>
		/// <returns></returns>
		bool TryGetLocalizedStrings( IPluginClient pluginClient, int count, out string[] arrStrings );




		/// <summary>
		/// Tries to get selected file/folder items in the current folder view. 
		/// </summary>
		/// <param name="selectedItems"></param>
		/// <returns>Returns true if successful, or false otherwise.</returns>
		bool TryGetSelection( out Address[] selectedItems );

		/// <summary>
		/// Tries to set selection in the current folder view.
		/// </summary>
		/// <param name="itemsToSelect">
		/// Addresses to select.
		/// The first one in the array will get focus.
		/// </param>
		/// <param name="fDeselectOthers">
		/// If this parameter is true, items which are not included in itemsToSelect will be deselected.
		/// If false, selections will be preserved.
		/// </param>
		/// <returns>Returns true if at least one item selected successfully, or false otherwise. </returns>
		bool TrySetSelection( Address[] itemsToSelect, bool fDeselectOthers );




		/// <summary>
		/// Create a new tab.
		/// </summary>
		/// <param name="address">address of new tab.</param>
		/// <param name="index">
		/// zero based index of insertion position.
		/// if this parameter is -1, new tab will be inserted at user setting position.
		/// </param>
		/// <param name="fLocked">if new tab is locked.</param>
		/// <param name="fSelect">if activate new tab.</param>
		/// <returns>Returns true if successful, or false otherwise.</returns>
		bool CreateTab( Address address, int index, bool fLocked, bool fSelect );
		

		/// <summary>
		/// Retrieves the current tabs.
		/// Note that returned array members may be disposed unexpectedly by user,
		/// so do not hold returned ITab inteterfaces but use this property as needed.
		/// </summary>
		ITab[] GetTabs();


		/// <summary>
		/// Gets or sets current active tab.
		/// </summary>
		ITab SelectedTab
		{
			get;
			set;
		}


		/// <summary>
		/// Retrieves a tab at the specified point in screen coordinates.
		/// </summary>
		/// <param name="pnt">point in screen corrdinates.</param>
		/// <returns>ITab interface if tab exists at pnt, or null otherwise.</returns>
		ITab HitTest( Point pnt );


		/// <summary>
		/// Open a new window of the specified address.
		/// </summary>
		/// <param name="address">Address to open.</param>
		/// <returns>Returns true if successful, or false otherwise.</returns>
		bool CreateWindow( Address address );

		
		/// <summary>
		/// Create and add tabs contained in groups.
		/// </summary>
		/// <param name="groupNames"></param>
		void OpenGroup( string[] groupNames );


		/// <summary>
		/// Gets group names that user registered.
		/// </summary>
		string[] Groups
		{
			get;
		}

		
		/// <summary>
		/// Gets group member paths.
		/// </summary>
		/// <param name="groupName"></param>
		/// <returns>null if groupName is invalid.</returns>
		string[] GetGroupPaths( string groupName );


		/// <summary>
		/// Adds paths to the specified group.
		/// if group specified by groupName exists, adds paths to the group.
		/// if not, creates a new group with paths.
		/// </summary>
		/// <param name="groupName"></param>
		/// <param name="paths"></param>
		/// <returns>Returns true if successful, or false otherwise.</returns>
		bool AddGroup( string groupName, string[] paths );


		/// <summary>
		/// Deletes a group.
		/// </summary>
		/// <param name="groupName"></param>
		/// <returns>Returns true if successful, or false otherwise.</returns>
		bool RemoveGroup( string groupName );



		/// <summary>
		/// NOT IMPLEMENTED
		/// </summary>
		/// <param name="name">
		/// Display name of application.
		/// 
		/// name can be a tree-path separated by "\".
		///    ex: "folder1\folder2\SomeApplication"
		/// 
		/// if upper fragments of path do not exist, those will be created.
		/// 
		/// overwrites if already exists item specified by name.
		/// </param>
		/// <param name="startInfo"></param>
		/// <returns></returns>
		bool AddApplication( string name, ProcessStartInfo startInfo );


		/// <summary>
		/// NOT IMPLEMENTED
		/// </summary>
		/// <param name="name">
		/// see AddApplication
		/// </param>
		/// <returns></returns>
		bool RemoveApplication( string name );


		/// <summary>
		/// NOT IMPLEMENTED. RETURNS NULL.
		/// </summary>
		/// <param name="name">
		/// name can be a path.
		/// if name is null or empty, all top-level items will be enumerated.
		/// </param>
		/// <returns></returns>
		ProcessStartInfo[] GetApplications( string name );




		/// <summary>
		/// Gets the window handle of QTTabBar.
		/// </summary>
		IntPtr Handle
		{
			get;
		}

		
		/// <summary>
		/// Gets the window handle of Explorer.
		/// </summary>
		IntPtr ExplorerHandle
		{
			get;
		}

		
		/// <summary>
		/// Gets or sets TabBar settings.
		/// </summary>
		TabBarOption TabBarOption
		{
			get;
			set;
		}

	}
}
